@@ -139,6 +139,14 @@
 /* .IP "\fBtls_required_enable (yes)\fR"
 /*	Enable support for the "TLS-Required: no" message header, defined
 /*	in RFC 8689.
+/* .PP
+/*	Available in Postfix 3.11 and later:
+/* .IP "\fBrequiretls_redact_dsn (yes)\fR"
+/*	When sending a delivery status notification for an original
+/*	message received with the REQUIRETLS option, remove the original
+/*	message body (as if that message was received with "RET=HDRS") and
+/*	do not enforce REQUIRETLS (as if that message was received without
+/*	REQUIRETLS).
 /* FILES
 /*	/var/spool/postfix/bounce/* non-delivery records
 /*	/var/spool/postfix/defer/* non-delivery records
@@ -197,6 +205,7 @@
 #include <dsb_scan.h>
 #include <hfrom_format.h>
 #include <sendopts.h>
+#include <dsn_mask.h>
 
 /* Single-threaded server skeleton. */
 
@@ -219,6 +228,7 @@ char   *var_delay_rcpt;
 char   *var_bounce_tmpl;
 bool    var_threaded_bounce;
 char   *var_hfrom_format;		/* header_from_format */
+int     var_reqtls_redact_dsn;
 
  /*
   * We're single threaded, so we can avoid some memory allocation overhead.
@@ -313,6 +323,27 @@ static int bounce_append_proto(char *service_name, VSTREAM *client)
 				  &rcpt_buf->rcpt, &dsn_buf->dsn));
 }
 
+/* edit_notification_properties - bounce message filter */
+
+static void edit_notification_properties(int *sendopts, int *dsn_ret)
+{
+
+    /*
+     * If REQUIRETLS is requested, do not propagate "TLS-Required: no". See
+     * also down-stream code in bounce_header().
+     */
+    if (*sendopts & SOPT_REQUIRETLS_ESMTP)
+	*sendopts &= ~SOPT_REQUIRETLS_HEADER;
+
+    /*
+     * Redact delivery status notification for REQUIRETLS message.
+     */
+    if (var_reqtls_redact_dsn && (*sendopts & SOPT_REQUIRETLS_ESMTP)) {
+	*sendopts &= ~SOPT_REQUIRETLS_ESMTP;
+	*dsn_ret = DSN_RET_HDRS;
+    }
+}
+
 /* bounce_notify_proto - bounce_notify server protocol */
 
 static int bounce_notify_proto(char *service_name, VSTREAM *client,
@@ -370,9 +401,10 @@ static int bounce_notify_proto(char *service_name, VSTREAM *client,
 	bounce_cleanup_register(service_name, STR(queue_id));
 
     /*
-     * Do not enforce the REQUIRETLS verb for bounce messages.
+     * Handle REQUIRETLS etc. matters.
      */
-    sendopts &= ~SOPT_REQUIRETLS_ESMTP;
+    if (var_requiretls_enable && (sendopts & SOPT_REQUIRETLS_ALL))
+	edit_notification_properties(&sendopts, &dsn_ret);
 
     /*
      * Execute the request.
@@ -443,9 +475,10 @@ static int bounce_verp_proto(char *service_name, VSTREAM *client)
 	bounce_cleanup_register(service_name, STR(queue_id));
 
     /*
-     * Do not enforce the REQUIRETLS verb for bounce messages.
+     * Handle REQUIRETLS etc. matters.
      */
-    sendopts &= ~SOPT_REQUIRETLS_ESMTP;
+    if (var_requiretls_enable && (sendopts & SOPT_REQUIRETLS_ALL))
+        edit_notification_properties(&sendopts, &dsn_ret);
 
     /*
      * Execute the request. Fall back to traditional notification if a bounce
@@ -540,9 +573,10 @@ static int bounce_one_proto(char *service_name, VSTREAM *client)
 		 STR(dsn_buf->action), STR(dsn_buf->reason));
 
     /*
-     * Do not enforce the REQUIRETLS verb for bounce messages.
+     * Handle REQUIRETLS etc. matters.
      */
-    sendopts &= ~SOPT_REQUIRETLS_ESMTP;
+    if (var_requiretls_enable && (sendopts & SOPT_REQUIRETLS_ALL))
+        edit_notification_properties(&sendopts, &dsn_ret);
 
     /*
      * Execute the request.
@@ -714,6 +748,7 @@ int     main(int argc, char **argv)
     };
     static const CONFIG_NBOOL_TABLE nbool_table[] = {
 	VAR_THREADED_BOUNCE, DEF_THREADED_BOUNCE, &var_threaded_bounce,
+	VAR_REQTLS_REDACT_DSN, DEF_REQTLS_REDACT_DSN, &var_reqtls_redact_dsn,
 	0,
     };
 
