@@ -96,15 +96,6 @@ int     bounce_one_service(int flags, char *queue_name, char *queue_id,
 				    var_notify_classes);
     VSTRING *new_id = vstring_alloc(10);
 
-    /*
-     * If the original sender requested REQUIRETLS, return headers only, and
-     * do not enforce REQUIRETLS for the delivery status notification.
-     */
-    if ((sendopts & SOPT_REQUIRETLS_ESMTP) != 0) {
-	dsn_ret = DSN_RET_HDRS;
-	sendopts &= ~SOPT_REQUIRETLS_ESMTP;
-    }
-
     /*
      * Initialize. Open queue file, bounce log, etc.
      */
