@@ -14,35 +14,35 @@
 /*	const char *relay;
 /*	DSN	*dsn;
 /*
-/*	int	defer_flush(flags, queue, id, encoding, smtputf8, sender,
+/*	int	defer_flush(flags, queue, id, encoding, sendopts, sender,
 /*				dsn_envid, dsn_ret)
 /*	int	flags;
 /*	const char *queue;
 /*	const char *id;
 /*	const char *encoding;
-/*	int	smtputf8;
+/*	int	sendopts;
 /*	const char *sender;
 /*	const char *dsn_envid;
 /*	int	dsn_ret;
 /*
-/*	int	defer_warn(flags, queue, id, encoding, smtputf8, sender,
+/*	int	defer_warn(flags, queue, id, encoding, sendopts, sender,
 				dsn_envid, dsn_ret)
 /*	int	flags;
 /*	const char *queue;
 /*	const char *id;
 /*	const char *encoding;
-/*	int	smtputf8;
+/*	int	sendopts;
 /*	const char *sender;
 /*	const char *dsn_envid;
 /*	int	dsn_ret;
 /*
-/*	int	defer_one(flags, queue, id, encoding, smtputf8, sender,
+/*	int	defer_one(flags, queue, id, encoding, sendopts, sender,
 /*				dsn_envid, ret, stats, recipient, relay, dsn)
 /*	int	flags;
 /*	const char *queue;
 /*	const char *id;
 /*	const char *encoding;
-/*	int	smtputf8;
+/*	int	sendopts;
 /*	const char *sender;
 /*	const char *dsn_envid;
 /*	int	dsn_ret;
@@ -128,8 +128,8 @@
 /*	Delivery status. See dsn(3). The specified action is ignored.
 /* .IP encoding
 /*	The body content encoding: MAIL_ATTR_ENC_{7BIT,8BIT,NONE}.
-/* .IP smtputf8
-/*	The level of SMTPUTF8 support (to be defined).
+/* .IP sendopts
+/*	Sender-requested SMTPUTF8 or RequireTLS support.
 /* .IP sender
 /*	The sender envelope address.
 /* .IP dsn_envid
@@ -153,6 +153,14 @@
 /*	IBM T.J. Watson Research
 /*	P.O. Box 704
 /*	Yorktown Heights, NY 10598, USA
+/*
+/*	Wietse Venema
+/*	Google, Inc.
+/*	111 8th Avenue
+/*	New York, NY 10011, USA
+/*
+/*	Wietse Venema
+/*	porcupine.org
 /*--*/
 
 /* System library. */
@@ -261,8 +269,8 @@ int     defer_append_intern(int flags, const char *id, MSG_STATS *stats,
 			   SEND_ATTR_INT(MAIL_ATTR_NREQ, BOUNCE_CMD_APPEND),
 				SEND_ATTR_INT(MAIL_ATTR_FLAGS, flags),
 				SEND_ATTR_STR(MAIL_ATTR_QUEUEID, id),
-				SEND_ATTR_FUNC(rcpt_print, (void *) rcpt),
-				SEND_ATTR_FUNC(dsn_print, (void *) &my_dsn),
+			    SEND_ATTR_FUNC(rcpt_print, (const void *) rcpt),
+			  SEND_ATTR_FUNC(dsn_print, (const void *) &my_dsn),
 				ATTR_TYPE_END) != 0)
 	    msg_warn("%s: %s service failure", id, var_defer_service);
 	log_adhoc(id, stats, rcpt, relay, &my_dsn, "deferred");
@@ -295,7 +303,7 @@ int     defer_append_intern(int flags, const char *id, MSG_STATS *stats,
 /* defer_flush - flush the defer log and deliver to the sender */
 
 int     defer_flush(int flags, const char *queue, const char *id,
-		            const char *encoding, int smtputf8,
+		            const char *encoding, int sendopts,
 		            const char *sender, const char *dsn_envid,
 		            int dsn_ret)
 {
@@ -308,7 +316,7 @@ int     defer_flush(int flags, const char *queue, const char *id,
 			    SEND_ATTR_STR(MAIL_ATTR_QUEUE, queue),
 			    SEND_ATTR_STR(MAIL_ATTR_QUEUEID, id),
 			    SEND_ATTR_STR(MAIL_ATTR_ENCODING, encoding),
-			    SEND_ATTR_INT(MAIL_ATTR_SMTPUTF8, smtputf8),
+			    SEND_ATTR_INT(MAIL_ATTR_SENDOPTS, sendopts),
 			    SEND_ATTR_STR(MAIL_ATTR_SENDER, sender),
 			    SEND_ATTR_STR(MAIL_ATTR_DSN_ENVID, dsn_envid),
 			    SEND_ATTR_INT(MAIL_ATTR_DSN_RET, dsn_ret),
@@ -323,7 +331,7 @@ int     defer_flush(int flags, const char *queue, const char *id,
  * do not flush the log */
 
 int     defer_warn(int flags, const char *queue, const char *id,
-		           const char *encoding, int smtputf8,
+		           const char *encoding, int sendopts,
 		         const char *sender, const char *envid, int dsn_ret)
 {
     if (mail_command_client(MAIL_CLASS_PRIVATE, var_defer_service,
@@ -333,7 +341,7 @@ int     defer_warn(int flags, const char *queue, const char *id,
 			    SEND_ATTR_STR(MAIL_ATTR_QUEUE, queue),
 			    SEND_ATTR_STR(MAIL_ATTR_QUEUEID, id),
 			    SEND_ATTR_STR(MAIL_ATTR_ENCODING, encoding),
-			    SEND_ATTR_INT(MAIL_ATTR_SMTPUTF8, smtputf8),
+			    SEND_ATTR_INT(MAIL_ATTR_SENDOPTS, sendopts),
 			    SEND_ATTR_STR(MAIL_ATTR_SENDER, sender),
 			    SEND_ATTR_STR(MAIL_ATTR_DSN_ENVID, envid),
 			    SEND_ATTR_INT(MAIL_ATTR_DSN_RET, dsn_ret),
@@ -347,7 +355,7 @@ int     defer_warn(int flags, const char *queue, const char *id,
 /* defer_one - defer mail for one recipient */
 
 int     defer_one(int flags, const char *queue, const char *id,
-		          const char *encoding, int smtputf8,
+		          const char *encoding, int sendopts,
 		          const char *sender, const char *dsn_envid,
 		          int dsn_ret, MSG_STATS *stats, RECIPIENT *rcpt,
 		          const char *relay, DSN *dsn)
@@ -369,7 +377,7 @@ int     defer_one(int flags, const char *queue, const char *id,
     if (delivery_status_filter != 0
     && (dsn_res = dsn_filter_lookup(delivery_status_filter, &my_dsn)) != 0) {
 	if (dsn_res->status[0] == '5')
-	    return (bounce_one_intern(flags, queue, id, encoding, smtputf8,
+	    return (bounce_one_intern(flags, queue, id, encoding, sendopts,
 				      sender, dsn_envid, dsn_ret, stats,
 				      rcpt, relay, dsn_res));
 	my_dsn = *dsn_res;
