@@ -33,7 +33,6 @@
 #include <dsn_buf.h>
 #include <header_body_checks.h>
 #include <sendopts.h>
-#include <domain_list.h>
 
  /*
   * Postfix TLS library.
@@ -45,6 +44,11 @@
   */
 #include <tls_proxy.h>
 
+ /*
+  * This application.
+  */
+#include <smtp_reqtls_policy.h>
+
  /*
   * Global iterator support. This is updated by the connection-management
   * loop, and contains dynamic context that appears in lookup keys for SASL
@@ -202,7 +206,7 @@ typedef struct SMTP_STATE {
 #ifdef USE_TLSRPT
     struct TLSRPT_WRAPPER *tlsrpt;
 #endif
-    int     enforce_requiretls;		/* from smtp_enforce_requiretls */
+    int     enforce_requiretls;		/* from smtp_reqtls_policy */
 #endif
 
     /*
@@ -370,7 +374,7 @@ extern STRING_LIST *smtp_use_srv_lookup;/* services with SRV record lookup */
 
 extern TLS_APPL_STATE *smtp_tls_ctx;	/* client-side TLS engine */
 extern int smtp_tls_insecure_mx_policy;	/* DANE post insecure MX? */
-extern DOMAIN_LIST *smtp_enforce_requiretls;	/* parsed list */
+extern SMTP_REQTLS_POLICY *smtp_reqtls_policy;	/* parsed list */
 
 #endif
 
