@@ -106,7 +106,6 @@
 #include <mail_addr.h>
 #include <valid_hostname.h>
 #include <sendopts.h>
-#include <domain_list.h>
 
 /* DNS library. */
 
@@ -516,30 +515,36 @@ static int smtp_get_effective_tls_level(DSN_BUF *why, SMTP_STATE *state)
     }
 
     /*
-     * If the sender requires TLS with server certificate verification, the
-     * TLS level must enforce a server certificate match.
+     * If REQUIRETLS is enforced, the TLS level must enforce a server
+     * certificate match. With opportunistic REQUIRETLS we connect to a
+     * server first.
      */
-    else if (var_requiretls_enable
-	     && (state->request->sendopts & SOPT_REQUIRETLS_ESMTP)) {
-	if (TLS_MUST_MATCH(tls->level) == 0) {
-	    if (state->enforce_requiretls) {
-		dsb_simple(why, "5.7.10", "REQUIRETLS Failure: Sender "
-			   "requires a TLS server certificate match, "
-			   "but the configured %s TLS security level '%s' "
-			   "does not support that. The last attempted "
-			   "server was %s",
-			   var_mail_name, str_tls_level(tls->level),
-			   STR(iter->host));
-		return (0);
-	    } else {
-		msg_info("REQUIRETLS Debug: Sender requires a TLS server "
-			 "certificate match, but the configured %s TLS "
-			 "security level '%s' does not support that. "
-			 "The last attempted server was %s",
-			 var_mail_name, str_tls_level(tls->level),
-			 STR(iter->host));
-		return (0);
-	    }
+    if (TLS_MUST_MATCH(tls->level) == 0) {
+	switch (state->enforce_requiretls) {
+	case SMTP_REQTLS_POLICY_ACT_ENFORCE:
+	    dsb_simple(why, "5.7.10", "REQUIRETLS Failure: sender "
+		       "requested REQUIRETLS, but the configured "
+		       "TLS security level '%s' does not support "
+		       "certificate matching. The last attempted "
+		       "server was %s", str_tls_level(tls->level),
+		       STR(iter->host));
+	    return (0);
+	case SMTP_REQTLS_POLICY_ACT_OPP_TLS:
+	    msg_info("%s: REQUIRETLS Debug: sender requested "
+		     "REQUIRETLS, but the configured TLS security "
+		     "level '%s' does not support certificate matching. "
+		     "The last attempted server was %s",
+		     state->request->queue_id,
+		     str_tls_level(tls->level), STR(iter->host));
+	    break;
+	case SMTP_REQTLS_POLICY_ACT_OPPORTUNISTIC:
+	case SMTP_REQTLS_POLICY_ACT_DISABLE:
+	    break;
+	default:
+	    dsb_simple(why, "4.7.10", "REQUIRETLS Failure: policy "
+		       "configuration error. The last attempted "
+		       "server was %s", STR(iter->host));
+	    return (0);
 	}
     }
 
@@ -579,15 +584,16 @@ static void smtp_connect_local(SMTP_STATE *state, const char *path)
 	SET_SCACHE_REQUEST_NEXTHOP(state, path);
 
     /*
-     * REQUIRETLS enforcement is based on the UNIX-domain pathname, without
-     * the "unix:" prefix.
+     * REQUIRETLS policy selection is based on the same TLS net-hop name as
+     * with certificate matching.
      */
 #ifdef USE_TLS
-    state->enforce_requiretls =
-	(var_requiretls_enable
-	 && smtp_enforce_requiretls
-	 && (state->request->sendopts & SOPT_REQUIRETLS_ESMTP) != 0
-	 && domain_list_match(smtp_enforce_requiretls, path));
+    if (smtp_reqtls_policy
+	&& STATE_REQTLS_IS_REQUESTED(var_requiretls_enable, state))
+	state->enforce_requiretls =
+	    smtp_reqtls_policy_eval(smtp_reqtls_policy, var_myhostname);
+    else
+	state->enforce_requiretls = SMTP_REQTLS_POLICY_ACT_DISABLE;
 #endif
 
     /*
@@ -1001,15 +1007,16 @@ static void smtp_connect_inet(SMTP_STATE *state, const char *nexthop,
 #endif						/* USE_TLSRPT */
 
 	/*
-	 * REQUIRETLS enforcement is based on the next-hop domain name
-	 * without the service or port.
+	 * REQUIRETLS policy selection is based on the same next-hop domain
+	 * name without service or port, as with certificate matching.
 	 */
 #ifdef USE_TLS
-	state->enforce_requiretls =
-	    (var_requiretls_enable
-	     && smtp_enforce_requiretls
-	     && (state->request->sendopts & SOPT_REQUIRETLS_ESMTP) != 0
-	     && domain_list_match(smtp_enforce_requiretls, domain));
+	if (smtp_reqtls_policy
+	    && STATE_REQTLS_IS_REQUESTED(var_requiretls_enable, state))
+	    state->enforce_requiretls =
+		smtp_reqtls_policy_eval(smtp_reqtls_policy, domain);
+	else
+	    state->enforce_requiretls = SMTP_REQTLS_POLICY_ACT_DISABLE;
 #endif
 
 	/*
