@@ -26,15 +26,26 @@ extern SMTP_REQTLS_POLICY *smtp_reqtls_policy_parse(const char *, const char *);
 extern int smtp_reqtls_policy_eval(SMTP_REQTLS_POLICY *, const char *);
 
 #define SMTP_REQTLS_POLICY_NAME_ENFORCE		"enforce"
-#define SMTP_REQTLS_POLICY_NAME_BEST_EFFORT	"best-effort"
+#define SMTP_REQTLS_POLICY_NAME_OPP_TLS		"opportunistic+starttls"
+#define SMTP_REQTLS_POLICY_NAME_OPPORTUNISTIC	"opportunistic"
 #define SMTP_REQTLS_POLICY_NAME_DISABLE		"disable"
 #define SMTP_REQTLS_POLICY_NAME_ERROR		"error"
 
-#define SMTP_REQTLS_POLICY_ACT_ENFORCE		2
-#define SMTP_REQTLS_POLICY_ACT_BEST_EFFORT	1
+#define SMTP_REQTLS_POLICY_ACT_ENFORCE		3
+#define SMTP_REQTLS_POLICY_ACT_OPP_TLS		2
+#define SMTP_REQTLS_POLICY_ACT_OPPORTUNISTIC	1
 #define SMTP_REQTLS_POLICY_ACT_DISABLE		0
 #define SMTP_REQTLS_POLICY_ACT_ERROR		(-1)
 
+#define STATE_REQTLS_IS_REQUESTED(var, state) \
+	SENDOPTS_REQTLS_IS_REQUESTED((var), (state)->request->sendopts)
+
+#define SENDOPTS_REQTLS_IS_REQUESTED(var, sendopts) \
+	((var) && (sendopts) & SOPT_REQUIRETLS_ESMTP)
+
+#define TLS_REQUIRED_BY_REQTLS_POLICY(enforce_requiretls) \
+	((enforce_requiretls) >= SMTP_REQTLS_POLICY_ACT_OPP_TLS)
+
 /* LICENSE
 /* .ad
 /* .fi
